/*
 * map.c
 */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "fplot.h"

void drawlati(double);
void drawlongi(double);
void pinit(double);
void pglobe(void);
void pmove(double, double);
void pcont(double, double);
void pnorm(double *);
void pmove1(double, double);
void pcont1(double, double);
void pconv(double, double, int *, int *);

enum status {off, on};

#define DEFAULT_CENTER 135
#define LEN 50

#ifdef UX700
char filename[] = "/usr1/bin/src/mapdata";
#endif

#ifdef SUN
char filename[] = "/home1/grp1/re00001/xdemo/c/mapdata";
#endif

#ifndef SUN
#ifndef UX700
char filename[] = "mapdata";
#endif
#endif

#ifdef __CYGWIN__
int main()
#else
int main(int argc, char **argv)
#endif
{
        int i;
        double la, lo;
        char line[LEN];
        FILE *fp;
        enum status pen;

#ifdef __CYGWIN__
	pinit(DEFAULT_CENTER);
#else
	pinit((argc > 1) ? atof(argv[1]) : DEFAULT_CENTER);
#endif
	pglobe();
        for (i = -90; i <= 90; i += 30) 
        	drawlati((double)i);
        for (i = 0; i < 360; i += 30) 
        	drawlongi((double)i);
        if ((fp = fopen(filename,"r")) == NULL) {
	    perror(filename);
	    return 1;
	}
        pen = off;
        while (fgets(line,LEN,fp), !feof(fp)) {
                if (*line == '"') 
                	pen = off;
                else {
                        sscanf(line,"%lf%lf",&lo,&la);
                        if (pen == off) {
                                pmove(la,lo);
                                pen = on;
                        }
                        else    pcont(la,lo);
                }
        }
	move( -200, 1000); label("<<WORLD MAP>>");
        mkplot("map.plot");
	closepl();
	return 0;
}

void drawlati(double la)
{
        pmove(la, 0.0);
	pcont(la, 90.0);
	pcont(la, 180.0);
	pcont(la, 270.0);
        pcont(la, 360.0);
}

void drawlongi(double lo)
{
        int i;
        pmove(90.0,lo);
        for (i = 85; i >= -90; i -= 5) 
                pcont((double)i,lo);
}

#define SCALE 10
#ifndef M_PI
#define M_PI 3.141592653589793
#endif
#define degtorad(deg) ((deg)/180.0*M_PI)

double center,lastlati,lastlongi;

void pinit(double lo)
{
        center = lo;
        openpl();
        erase();
        space(-SCALE*181,-SCALE*181,SCALE*181,SCALE*181);
}

void pglobe(void)
{
	int i;
	pmove1(90.0,center-180.0);
	for (i = 85; i >= -90; i -= 5) 
		pcont1((double)i,center-180.0);
	for (i = -85; i <= 90; i += 5)
		pcont1((double)i,center+180.0);
	lastlati = 90.0;
	lastlongi = center+180.0;
}

void pmove(double la, double lo)
{
	pnorm(&lo);
        pmove1(la,lo);
        lastlati = la;
        lastlongi = lo;
}

void pcont(double la, double lo)
{
        double dir,ghostlo,crossla;
	pnorm(&lo);
        if (fabs(lo-lastlongi) > 180.0) {
		dir = (lo < lastlongi) ? 1.0 : -1.0;
                ghostlo = lo+dir*360.0;
		if (ghostlo == lastlongi) 
			crossla = la;
		else 
                	crossla = lastlati+
                		(la-lastlati)/(ghostlo-lastlongi)*
                		(center+dir*180.0-lastlongi);
                pcont1(crossla,center+dir*180.0);
                pmove1(crossla,center-dir*180.0);
        }
        pcont1(la,lo);
        lastlati = la;
        lastlongi = lo;
}

void pnorm(double *lop)
{
	double x;
        x = fmod(*lop-center,360.0)+center;
        if (x > center+180.0) 
        	*lop = x-360.0;
        else if (x < center-180.0) 
        	*lop = x+360.0;
	else 	*lop = x;
}

void pmove1(double la, double lo)
{
	int x,y;
	pconv(la,lo,&x,&y);
	move(x,y);
}

void pcont1(double la, double lo)
{
	int x,y;
	pconv(la,lo,&x,&y);
	cont(x,y);
}

void pconv(double la, double lo, int *xp, int *yp)
{
        *yp = (int)((double)SCALE*la);
        *xp = (int)((double)SCALE*(lo-center)*cos(degtorad(la)));
}
